/*
 * pif_Iterator is a protocol used to implemented Iterators. Iterators are objects that traverse
 * incrementally through a collection of data in some stated order. For example, an iterator that
 * loops through a list would probably go from the first element to the last, while a *reverse*
 * iterator would go from the last element to the first. For an example of these, see the provided
 * /ForwardListIterator and /ReverseListIterator classes.
 *
 * Because this is just a protocol, it implements only the 'protocol class' that others extend, as
 * well as methods that must be implemented (to at least some extent).
 */

#if !defined(PIF_NOPREFIX_GENERAL) && !defined(PIF_NOPREFIX_ITERATOR)
pif_Iterator
#else
Iterator
#endif
	New(data)
		/*

		Behavior.

		  Instanstiates the Iterator object.

		Argument.

		  data. The source or information the Iterator is using to collect data from.

		*/

	proc
		Initialize(/* argument (optional) */)
			/*

			Behavior.

			  Two possible behaviors:

				1. When first called, Initialize() initializes the Iterator object (with a possible
                   argument; see Arguments) so that the object can begin iterating through the data.

				2. If called a second (or third, or etc.) time, then these calls should be
                   interpreted as trying to restart the iterator (again, with a possible argument).
                   This behavior is allowed to fail (see Returns), which may be required if, for
                   example, the data that was iterated through is no longer available.

			Arguments.

			  (optional) Initializes the Iterator with some value. This could be, for example, a
                         setting for the Iterator object or some position to start at.

			  Note: Such a setting should note impact the order in which the Iterator looks through
                    data, but it can impact how it 'steps' through it. For example, if you're
                    iterating through a string then the argument can specify, say, a delimiter. But
                    it shouldn't specify that you go backwards through the string rather than
                    forwards, or alternate from the first element, then the last, then the second,
                    then second to last, or etc. This should be done with another Iterator class.

			        Essentially, the manner in which the Iterator goes through the data should be
                    consistent regardless of the argument.

			  Note: Initialize() should never *require* an argument by the end-programer. Any
                    potential settings should take a default value, and there should always be a
                    default 'start' position.

			Returns.

			  1 upon successful initialization, and 0 otherwise.

			  Initialization may fail if, for example, a supplied argument is invalid (an invalid
			  option or position outside the bounds of the data) or if Initialize() was used to try
              and restart the Iterator, but the Iterator object can not be restarted.

			Implementation.

			  Required.

			Note: The behavior of all other methods is undefined if Initialize() is not called first.

			*/

			return 0

		Next(/* position (optional) */)
			/*

			Behavior.

			  Moves the position of the Iterator object to the next position.

			  Note: Next() and Step(1) (see below) should have the same behavior.

			Arguments.

			  position. (optional) Indicates the position in the data to move to.

			Returns.

			  Numeric value of the position within the data, or 0 when the end of the data is
              reached.

			Implementation.

			  Required.

			*/

			return 0

		Step(step)
			/*

			Behavior.

			  Moves the position of the Iterator object forwards a specified amount.

			  Note: Step(1) and Next() (see above) should have the same behavior.

			Arguments.

			  step. The number of positions to move the data. Positive numbers indicate moving the
                    position in the data forwards, and negative number indicate moving it backwards.

			        Note: If step = 1, then Next() and Step(step) should have the same behavior.

			  Returns.

			    Numeric value of the position within the data, or 0 when the end of the data is
                reached.

			  Implementation.

			    Optional.

			*/

			return 0

		Look()
			/*

			Behavior.

			  Gets the data at the current position.

			  Note: Look() and Peek(0) (see below) should have the same behavior.

			Returns.

			  The data at the current position, or null if the end of the data has been reached.

			Implementation.

			  Required.

			*/

			return null

		Peek(step = 1)
			/*

			Behavior.

			  Looks ahead (or behind) the amount specified by step.

			Arguments.

			  step (default: 1). If step > 0, then output the data at Position() + step. If step <
                   0, the output the data at Position() - Abs(step) (i.e., move backwards that
                   amount).

			       Note: If step = 0, then Peek(step) and Look() (see above) have the same
			             behavior.

			Returns.

			  The data at the specified position, or null if that data is unavailable. Data may be
              unavailable if it is, for example, outside the bounds of a given data structure being
              iterated through, or the data forwards and/or backwards is no longer available.

			  Note: If the Iterator has reached a point where Done() is true (i.e., it is finished
                    looking through the data), Peek() should return null for all arguments.

			Implementation.

			  Optional.

			*/

			return null

		Done()
			/*

			Behavior.

			  Indicates whether the Iterator object has reached the end of the data.

			Returns.

			  True when the end of the data has been reached, and false otherwise.

			Implementation.

			  Required.

			*/

			return 1

		Position()
			/*

			Behavior.

			  Outputs the position of the data. If the data is not in an order that a position makes
              sense (for example, if the data is streaming from some source and then discarded), it
              may act as a counter instead.

			Returns.

			  The position of the data, or the counter for how many times the Iterator object has
              been moved forwards.

			Implementation.

			  Optional.

			*/

			return 0